class Logger {
    constructor(debug) {
        this.debug = debug;
    }
    error(message) {
        if (this.debug) {
            throw Error(message);
        }
        else {
            console.warn(message);
        }
    }
}

const FLAG_NAME = 'allow-property-update';
const VALUE_PREFIX = 'stateless-';
function watcher(fn, target) {
    return function (...args) {
        target[FLAG_NAME] = true;
        let result = fn.apply(target, ...args);
        return result;
    };
}
/**
 * Adds watcher to the methods.
 * @param target
 * @param instance
 */
function addPrototypeWatcher(target, instance, options) {
    let exclude = [...["constructor", "ngOnInit"], ...options.excludeMethods];
    let props = target.prototype;
    let fns = Object.getOwnPropertyNames(props).filter(x => !exclude.includes(x));
    fns.forEach(fn => {
        let oldFn = target.prototype[fn];
        target.prototype[fn] = watcher(oldFn, instance);
    });
}
/**
 * Adds property descriptor.
 * @param inputs
 * @param self
 * @param target
 */
function addPropertyDesc(inputs, self, target, options, logger) {
    for (let input in inputs) {
        Object.defineProperty(self, input, {
            get: function () {
                return self[VALUE_PREFIX + input + 'value'];
            },
            set: function (e) {
                if (self[FLAG_NAME]) {
                    self[FLAG_NAME] = false;
                    logger.error(`Cannot modify input property ${input} since the class ${target.name} is marked stateless.`);
                    return;
                }
                self[VALUE_PREFIX + input + 'value'] = e;
            }
        });
    }
}
function toInternalOptions(op = {}) {
    const def = { debug: true, excludeMethods: [] };
    for (var prop in def) {
        if (op[prop] !== undefined) {
            def[prop] = op[prop];
        }
    }
    return def;
}

/**
 * Makes an Angular component stateless.
 */
function StatelessComponent(options) {
    const opts = toInternalOptions(options);
    const logger = new Logger(opts.debug);
    return function Def(target) {
        if (!target.ɵcmp) {
            logger.error(`${target.name} is not an Angular component. Add @Component decorator to the ${target.name}`);
            return;
        }
        let inputs = target.ɵcmp.inputs;
        return class Extender extends target {
            constructor(...args) {
                super(...args);
                let self = this;
                addPrototypeWatcher(target, self, opts);
                addPropertyDesc(inputs, self, target, opts, logger);
            }
            ;
        };
    };
}

/*
 * Public API Surface of angular-decorator
 */

/**
 * Generated bundle index. Do not edit.
 */

export { FLAG_NAME, StatelessComponent, VALUE_PREFIX, addPropertyDesc, addPrototypeWatcher, toInternalOptions };
//# sourceMappingURL=protools-angular.js.map
